package com.jkcredit.asychronous.dao;

import com.jkcredit.asychronous.entity.Charge;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.repository.CountQuery;
import org.springframework.data.mongodb.repository.MongoRepository;
import org.springframework.data.mongodb.repository.Query;

import java.util.List;

public interface ChargeRepository extends MongoRepository<Charge, Long> {

    /**
     * 查找计费记录
     * @param userName 用户名
     * @param api api名称
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param isCharge 是否计费
     * @return 返回查询列表
     */
    @Query(value = "{" +
            "    userName:?0,\n" +
            "    api:?1,\n" +
            "    time:{$gt:?2,$lt:?3},\n" +
            "    isCharge:?4\n" +
            "}")
    List<Charge> findByUserAndTime(String userName, String api, Long startTime, Long endTime,Integer isCharge);

    /**
     * 条件查询计费条数
     * @param userName 用户名
     * @param api api名称
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param isCharge 是否计费
     * @return 返回条数
     */
    @CountQuery(value = "{" +
            "    userName:?0,\n" +
            "    api:?1,\n" +
            "    time:{$gt:?2,$lt:?3},\n" +
            "    isCharge:?4\n" +
            "}")
    int findCountByUserAndTime(String userName, String api, Long startTime, Long endTime,Integer isCharge);

    /**
     * 根据api实际查询计费量
     * @param api api名称
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param isCharge 是否计费
     * @return 返回条数
     */
    @CountQuery(value = "{" +
            "    api:?0,\n" +
            "    time:{$gt:?1,$lt:?2},\n" +
            "    isCharge:?3\n" +
            "}")
    int findCountByTime(String api, Long startTime, Long endTime,Integer isCharge);

//    Page<Charge> find(Criteria criteria, Pageable pageable);
}
