package com.jkcredit.asychronous.process;

import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.asychronous.common.ResponseEntity;
import com.jkcredit.asychronous.entity.Charge;
import com.jkcredit.asychronous.entity.Man;
import com.jkcredit.asychronous.entity.ManFtpResult;
import com.jkcredit.asychronous.entity.ManResult;
import com.jkcredit.asychronous.service.ChargeService;
import com.jkcredit.asychronous.service.ManFtpResultService;
import com.jkcredit.asychronous.service.ManResultService;
import com.jkcredit.asychronous.service.ManService;
import com.jkcredit.asychronous.util.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.component.file.GenericFileMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import org.springframework.http.HttpEntity;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Component;

import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;

/**
 * @Descirption
 * @Author Zhaoning Sun
 * @Date Created in 2018/6/22 下午7:46
 */
@Slf4j
@Component
public class ManLocationFileProcessor implements Processor {

    @Value("${manftp.download.info}")
    private String fileDir;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private ManResultService manResultService;

    @Autowired
    private ManService manService;

    @Autowired
    private ChargeService chargeService;

    @Autowired
    private ManFtpResultService manFtpResultService;

    @Override
    public void process(Exchange exchange) {
        exchange.setProperty(Exchange.CHARSET_NAME, "UTF-8");
        GenericFileMessage<RandomAccessFile> inFileMessage = (GenericFileMessage<RandomAccessFile>) exchange.getIn();
        //文件名
        String fileName = inFileMessage.getGenericFile().getFileName();
        //系统文件分隔符
        String splitTag = File.separator;
        //文件的绝对路径
        log.info("manProcess:" + fileDir + splitTag + fileName);

        Charge charge = new Charge();

        List<ManResult> manResultList = new ArrayList<>();
        //文件读取完返回结果
        List<ManFtpResult> manFtpResultList = ExcelUtil.readExcel(fileDir + splitTag + fileName);

        ManResult manResult = new ManResult();
        if (manFtpResultList.size() > 0) {
            manFtpResultService.insert(manFtpResultList);
            //根据文件名称获取时间
//            String[] beforStr = fileName.split("_");
            String[] str = fileName.split("\\.");
            fileName = str[0];
            Long beginTime = Long.parseLong(fileName) - 10000;
            Long endTime = Long.parseLong(fileName);
            //查询时间段内的数据
            for (ManFtpResult manFtpResult : manFtpResultList) {

                Man man = manService.selectById(manFtpResult.getManId());
                //从数据库查询的结果跟 excel结果进行比对
                manResult = new ManResult();
                String userName = man.getUserName();
                String name = man.getName();
                String idCard = man.getIdCard();
                String requestId = man.getRequestId();

                manResult.setName(name);
                manResult.setUserName(userName);
                manResult.setIdCard(idCard);
                manResult.setRequestId(requestId);

                //查询失败 设置1001
                manResult.setResultCode(1001);
                manResult.setCallBlackUrl(man.getCallBlackUrl());

                //如果身份证号码一致设置参数
                if (man.getIdCard().equals(manFtpResult.getIdCard()) && man.getName().equals(manFtpResult.getName())) {

                    String isIdCard = manFtpResult.getIsIdCard();
                    String isDrugs = manFtpResult.getIsDrugs();
                    String isEcape = manFtpResult.getIsEscape();
                    String isPedigree = manFtpResult.getIsPedigree();
                    manResult.setResultCode(1000);

                    //判断返回码是否为2 如果是2 设置状态码1002 不计费
                    if (("2").equals(isIdCard)) {
                        manResult.setResultCode(1002);
                    }
                    if (("2").equals(isDrugs)) {
                        manResult.setResultCode(1002);
                    }
                    if (("2").equals(isEcape)) {
                        manResult.setResultCode(1002);
                    }
                    if (("2").equals(isPedigree)) {
                        manResult.setResultCode(1002);
                    }

                    name = manFtpResult.getName();
                    manResult.setName(name);
                    manResult.setUserName(userName);
                    manResult.setIdCard(idCard);
                    manResult.setIsIdCard(stringForInt(isIdCard));
                    manResult.setIsEscape(stringForInt(isEcape));
                    manResult.setIsDrugs(stringForInt(isDrugs));
                    manResult.setIsPedigree(stringForInt(isPedigree));
                    //查询是成功设置1000
                }
                //设置返回批次
                manResult.setBatch(fileName);
                manResultList.add(manResult);
            }

            /**
             * 将返回结果返回给回调地址
             */
            if (manResultList.size() > 0) {
                manResultList.forEach(manResults -> {
                    //发送请求给回调地址
                    try {
                        HttpEntity httpEntity = restTemplate.postForEntity(manResults.getCallBlackUrl(), manResults,
                                String.class);
                        int stateCode = 200;
                        if (((org.springframework.http.ResponseEntity) httpEntity).getStatusCodeValue() == stateCode) {
                            charge.setUserName(manResults.getUserName());
                            charge.setApi("MAN_CHECK");
                            charge.setCode(manResults.getResultCode());
                            charge.setIsCharge(1);
                            charge.setCallBackUrl(manResults.getCallBlackUrl());
                            charge.setResult(manResults);
                            charge.setTime(System.currentTimeMillis());

                            String json = JSONObject.toJSONString(charge);
                            log.info(json);
                            chargeService.insert(charge);
                        }
                    } catch (Exception e) {
                        charge.setUserName(manResults.getUserName());
                        charge.setApi("MAN_CHECK");
                        charge.setCode(manResults.getResultCode());
                        charge.setIsCharge(1);
                        charge.setCallBackUrl(manResults.getCallBlackUrl());
                        charge.setResult(manResults);
                        charge.setTime(System.currentTimeMillis());
                        String json = JSONObject.toJSONString(charge);
                        log.info(json);
                        chargeService.insert(charge);
                        log.error("回调地址不通:{}", manResults.getCallBlackUrl());
                    }
                });
                manResultService.insert(manResultList);
            } else {
                log.error("error:{}", new ResponseEntity(400, "无结果", "erro", ""));
            }
        } else {
            log.error("error:{}", new ResponseEntity(401, "读取文件内容为空", "erro", ""));
        }


    }

    /**
     * 去读Excel的方法readExcel，该方法入口参数为一个Filedir
     *
     * @param file
     * @return
     */
    public List<ManFtpResult> readExcel(String file) {
        ImportParams params = new ImportParams();
        params.setHeadRows(0);
        List<ManFtpResult> list = ExcelImportUtil.importExcel(
                new File(file), ManFtpResult.class, params);
        return list;
    }

    public Integer stringForInt(String str) {
        return Integer.parseInt(str.substring(0, 1));
    }

    public static void main(String[] args) {

    }

}
