package com.jkcredit.asychronous.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

@Slf4j
public class FtpUtil {

    private FTPClient ftp;


    public FtpUtil(String addr, int port, String username, String password) throws IOException {
        ftp = new FTPClient();
        ftp.connect(addr, port);
        ftp.login(username, password);
        ftp.setFileType(FTPClient.BINARY_FILE_TYPE);
        int reply;
        reply = ftp.getReplyCode();
        if (!FTPReply.isPositiveCompletion(reply)) {
            ftp.disconnect();
        }
    }

    /**
     * @param path 上传到ftp服务器哪个路径下
     * @return
     * @throws Exception
     */
    public void connect(String path) throws Exception {
        ftp.changeWorkingDirectory(path);
    }

    public boolean makeDirectory(String path) throws IOException {
        return ftp.makeDirectory(path);
    }


    /**
     * @param file 上传的文件或文件夹
     * @throws Exception
     */
    public void upload(File file) throws Exception {
        if (file.isDirectory()) {
            ftp.makeDirectory(file.getName());
            ftp.changeWorkingDirectory(file.getName());
            String[] files = file.list();
            for (int i = 0; i < files.length; i++) {
                File file1 = new File(file.getPath() + File.separator + files[i]);
                if (file1.isDirectory()) {
                    upload(file1);
                    ftp.changeToParentDirectory();
                } else {
                    File file2 = new File(file.getPath() + File.separator + files[i]);
                    FileInputStream input = new FileInputStream(file2);
                    ftp.storeFile(file.getName(), input);
                    input.close();
                }
            }
            log.info("文件上传");
        } else {
            File file2 = new File(file.getPath());
            FileInputStream input = new FileInputStream(file2);
            ftp.enterLocalPassiveMode();
            boolean result = ftp.storeFile(file.getName(), input);
            input.close();
            log.info("文件上传:{}", result);
        }
    }

    public static void fileUpload(String host, Integer port, String userName, String passWord, String dir, String fileName) {
        try {
//            log.info("文件开始上传");
            FtpUtil ftp = new FtpUtil(host, port, userName, passWord);
            ftp.connect(dir);
            File file = new File(fileName);
            ftp.upload(file);
            ftp.destroy();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void destroy() throws IOException {
        if (ftp != null) {
            ftp.disconnect();
            ftp = null;
        }
    }
}